@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2018-2021 Mike Fährmann
+# Copyright 2018-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,6 +10,7 @@
 
 from .common import Extractor, Message
 from .. import text, util, exception
+import itertools
 import random
 import string
 
@@ -20,6 +21,7 @@ class ArtstationExtractor(Extractor):
     filename_fmt = "{category}_{id}_{asset[id]}_{title}.{extension}"
     directory_fmt = ("{category}", "{userinfo[username]}")
     archive_fmt = "{asset[id]}"
+    browser = "firefox"
     root = "https://www.artstation.com"
 
     def __init__(self, match):
@@ -30,16 +32,23 @@ class ArtstationExtractor(Extractor):
     def items(self):
         data = self.metadata()
 
-        for project in self.projects():
-            for asset in self.get_project_assets(project["hash_id"]):
+        projects = self.projects()
+        max_posts = self.config("max-posts")
+        if max_posts:
+            projects = itertools.islice(projects, max_posts)
+
+        for project in projects:
+            for num, asset in enumerate(
+                    self.get_project_assets(project["hash_id"]), 1):
                 asset.update(data)
                 adict = asset["asset"]
+                asset["num"] = num
                 yield Message.Directory, asset
 
                 if adict["has_embedded_player"] and self.external:
                     player = adict["player_embedded"]
-                    url = text.extract(player, 'src="', '"')[0] or \
-                        text.extract(player, "src='", "'")[0]
+                    url = (text.extr(player, 'src="', '"') or
+                           text.extr(player, "src='", "'"))
                     if url and not url.startswith(self.root):
                         asset["extension"] = None
                         yield Message.Url, "ytdl:" + url, asset
@@ -73,7 +82,12 @@ class ArtstationExtractor(Extractor):
     def get_project_assets(self, project_id):
         """Return all assets associated with 'project_id'"""
         url = "{}/projects/{}.json".format(self.root, project_id)
-        data = self.request(url).json()
+
+        try:
+            data = self.request(url).json()
+        except exception.HttpError as exc:
+            self.log.warning(exc)
+            return
 
         data["title"] = text.unescape(data["title"])
         data["description"] = text.unescape(text.remove_html(
@@ -84,6 +98,7 @@ class ArtstationExtractor(Extractor):
         assets = data["assets"]
         del data["assets"]
 
+        data["count"] = len(assets)
         if len(assets) == 1:
             data["asset"] = assets[0]
             yield data
@@ -99,16 +114,23 @@ class ArtstationExtractor(Extractor):
         return response.json()
 
     def _pagination(self, url, params=None, json=None):
+        headers = {
+            "Accept" : "application/json, text/plain, */*",
+            "Origin" : self.root,
+            "Referer": self.root + "/",
+        }
+
         if json:
             params = json
-            kwargs = {"json": json}
+            headers["PUBLIC-CSRF-TOKEN"] = self._init_csrf_token()
+            kwargs = {"method": "POST", "headers": headers, "json": json}
         else:
             if not params:
                 params = {}
-            kwargs = {"params": params}
+            kwargs = {"params": params, "headers": headers}
 
-        params["page"] = 1
         total = 0
+        params["page"] = 1
 
         while True:
             data = self.request(url, **kwargs).json()
@@ -120,6 +142,17 @@ class ArtstationExtractor(Extractor):
 
             params["page"] += 1
 
+    def _init_csrf_token(self):
+        url = self.root + "/api/v2/csrf_protection/token.json"
+        headers = {
+            "Accept" : "*/*",
+            "Origin" : self.root,
+            "Referer": self.root + "/",
+        }
+        return self.request(
+            url, method="POST", headers=headers, json={},
+        ).json()["public_csrf_token"]
+
     @staticmethod
     def _no_cache(url, alphabet=(string.digits + string.ascii_letters)):
         """Cause a cache miss to prevent Cloudflare 'optimizations'
@@ -146,14 +179,15 @@ class ArtstationUserExtractor(ArtstationExtractor):
                r"/(?!artwork|projects|search)([^/?#]+)(?:/albums/all)?"
                r"|((?!www)\w+)\.artstation\.com(?:/projects)?)/?$")
     test = (
-        ("https://www.artstation.com/gaerikim/", {
+        ("https://www.artstation.com/sungchoi/", {
             "pattern": r"https://\w+\.artstation\.com/p/assets/images"
                        r"/images/\d+/\d+/\d+/(4k|large|medium|small)/[^/]+",
-            "count": ">= 6",
+            "range": "1-10",
+            "count": ">= 10",
         }),
-        ("https://www.artstation.com/gaerikim/albums/all/"),
-        ("https://gaerikim.artstation.com/"),
-        ("https://gaerikim.artstation.com/projects/"),
+        ("https://www.artstation.com/sungchoi/albums/all/"),
+        ("https://sungchoi.artstation.com/"),
+        ("https://sungchoi.artstation.com/projects/"),
     )
 
     def projects(self):
@@ -293,34 +327,47 @@ class ArtstationSearchExtractor(ArtstationExtractor):
     archive_fmt = "s_{search[query]}_{asset[id]}"
     pattern = (r"(?:https?://)?(?:\w+\.)?artstation\.com"
                r"/search/?\?([^#]+)")
-    test = ("https://www.artstation.com/search?q=ancient&sort_by=rank", {
+    test = ("https://www.artstation.com/search?query=ancient&sort_by=rank", {
         "range": "1-20",
         "count": 20,
     })
 
     def __init__(self, match):
         ArtstationExtractor.__init__(self, match)
-        query = text.parse_query(match.group(1))
-        self.query = query.get("q", "")
-        self.sorting = query.get("sort_by", "rank").lower()
+        self.params = query = text.parse_query(match.group(1))
+        self.query = text.unquote(query.get("query") or query.get("q", ""))
+        self.sorting = query.get("sort_by", "relevance").lower()
+        self.tags = query.get("tags", "").split(",")
 
     def metadata(self):
         return {"search": {
             "query"  : self.query,
             "sorting": self.sorting,
+            "tags"   : self.tags,
         }}
 
     def projects(self):
+        filters = []
+        for key, value in self.params.items():
+            if key.endswith("_ids") or key == "tags":
+                filters.append({
+                    "field" : key,
+                    "method": "include",
+                    "value" : value.split(","),
+                })
+
         url = "{}/api/v2/search/projects.json".format(self.root)
-        return self._pagination(url, json={
-            "additional_fields": "[]",
-            "filters"          : "[]",
-            "page"             : None,
-            "per_page"         : "50",
-            "pro_first"        : "1",
+        data = {
             "query"            : self.query,
+            "page"             : None,
+            "per_page"         : 50,
             "sorting"          : self.sorting,
-        })
+            "pro_first"        : ("1" if self.config("pro-first", True) else
+                                  "0"),
+            "filters"          : filters,
+            "additional_fields": (),
+        }
+        return self._pagination(url, json=data)
 
 
 class ArtstationArtworkExtractor(ArtstationExtractor):
@@ -371,6 +418,10 @@ class ArtstationImageExtractor(ArtstationExtractor):
             "options": (("external", True),),
             "pattern": "ytdl:https://www.youtube.com/embed/JNFfJtwwrU0",
         }),
+        # 404 (#3016)
+        ("https://www.artstation.com/artwork/3q3mXB", {
+            "count": 0,
+        }),
         # alternate URL patterns
         ("https://sungchoi.artstation.com/projects/LQVJr"),
         ("https://artstn.co/p/LQVJr"),
@@ -384,7 +435,10 @@ class ArtstationImageExtractor(ArtstationExtractor):
     def metadata(self):
         self.assets = list(ArtstationExtractor.get_project_assets(
             self, self.project_id))
-        self.user = self.assets[0]["user"]["username"]
+        try:
+            self.user = self.assets[0]["user"]["username"]
+        except IndexError:
+            self.user = ""
         return ArtstationExtractor.metadata(self)
 
     def projects(self):
@@ -399,7 +453,7 @@ class ArtstationFollowingExtractor(ArtstationExtractor):
     subcategory = "following"
     pattern = (r"(?:https?://)?(?:www\.)?artstation\.com"
                r"/(?!artwork|projects|search)([^/?#]+)/following")
-    test = ("https://www.artstation.com/gaerikim/following", {
+    test = ("https://www.artstation.com/sungchoi/following", {
         "pattern": ArtstationUserExtractor.pattern,
         "count": ">= 50",
     })
