@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2022 Mike Fährmann
+# Copyright 2019-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,8 +9,7 @@
 """Extractors for Blogger blogs"""
 
 from .common import Extractor, Message
-from .. import text
-import json
+from .. import text, util
 import re
 
 BASE_PATTERN = (
@@ -48,6 +47,7 @@ class BloggerExtractor(Extractor):
             r'\d+\.bp\.blogspot\.com)/[^"]+)').findall
         findall_video = re.compile(
             r'src="(https?://www\.blogger\.com/video\.g\?token=[^"]+)').findall
+        metadata = self.metadata()
 
         for post in self.posts(blog):
             content = post["content"]
@@ -60,16 +60,13 @@ class BloggerExtractor(Extractor):
                 page = self.request(post["url"]).text
                 for url in findall_video(page):
                     page = self.request(url).text
-                    video_config = json.loads(text.extract(
-                        page, 'var VIDEO_CONFIG =', '\n')[0])
+                    video_config = util.json_loads(text.extr(
+                        page, 'var VIDEO_CONFIG =', '\n'))
                     files.append(max(
                         video_config["streams"],
                         key=lambda x: x["format_id"],
                     )["play_url"])
 
-            if not files:
-                continue
-
             post["author"] = post["author"]["displayName"]
             post["replies"] = post["replies"]["totalItems"]
             post["content"] = text.remove_html(content)
@@ -77,18 +74,21 @@ class BloggerExtractor(Extractor):
             del post["selfLink"]
             del post["blog"]
 
-            yield Message.Directory, {"blog": blog, "post": post}
-            for num, url in enumerate(files, 1):
-                yield Message.Url, url, text.nameext_from_url(url, {
-                    "blog": blog,
-                    "post": post,
-                    "url" : url,
-                    "num" : num,
-                })
+            data = {"blog": blog, "post": post}
+            if metadata:
+                data.update(metadata)
+            yield Message.Directory, data
+
+            for data["num"], url in enumerate(files, 1):
+                data["url"] = url
+                yield Message.Url, url, text.nameext_from_url(url, data)
 
     def posts(self, blog):
         """Return an iterable with all relevant post objects"""
 
+    def metadata(self):
+        """Return additional metadata"""
+
 
 class BloggerPostExtractor(BloggerExtractor):
     """Extractor for a single blog post"""
@@ -136,9 +136,9 @@ class BloggerPostExtractor(BloggerExtractor):
             "pattern": r"https://.+\.googlevideo\.com/videoplayback",
         }),
         # image URLs with width/height (#1061)
-        ("https://aaaninja.blogspot.com/2020/08/altera-boob-press-2.html", {
-            "pattern": r"https://1.bp.blogspot.com/.+/s0/altera_.+png",
-        }),
+        #  ("https://aaaninja.blogspot.com/2020/08/altera-boob-press-2.html", {
+        #      "pattern": r"https://1.bp.blogspot.com/.+/s0/altera_.+png",
+        #  }),
         # new image domain (#2204)
         (("https://randomthingsthroughmyletterbox.blogspot.com/2022/01"
           "/bitter-flowers-by-gunnar-staalesen-blog.html"), {
@@ -176,31 +176,48 @@ class BloggerBlogExtractor(BloggerExtractor):
 
 
 class BloggerSearchExtractor(BloggerExtractor):
-    """Extractor for search resuls and labels"""
+    """Extractor for Blogger search resuls"""
     subcategory = "search"
-    pattern = BASE_PATTERN + r"/search(?:/?\?q=([^/?#]+)|/label/([^/?#]+))"
+    pattern = BASE_PATTERN + r"/search/?\?q=([^&#]+)"
     test = (
         ("https://julianbphotography.blogspot.com/search?q=400mm", {
-            "count": "< 10"
-        }),
-        ("https://dmmagazine.blogspot.com/search/label/D%26D", {
-            "range": "1-25",
-            "count": 25,
+            "count": "< 10",
+            "keyword": {"query": "400mm"},
         }),
     )
 
     def __init__(self, match):
         BloggerExtractor.__init__(self, match)
-        query = match.group(3)
-        if query:
-            self.query, self.label = query, None
-        else:
-            self.query, self.label = None, match.group(4)
+        self.query = text.unquote(match.group(3))
 
     def posts(self, blog):
-        if self.query:
-            return self.api.blog_search(blog["id"], text.unquote(self.query))
-        return self.api.blog_posts(blog["id"], text.unquote(self.label))
+        return self.api.blog_search(blog["id"], self.query)
+
+    def metadata(self):
+        return {"query": self.query}
+
+
+class BloggerLabelExtractor(BloggerExtractor):
+    """Extractor for Blogger posts by label"""
+    subcategory = "label"
+    pattern = BASE_PATTERN + r"/search/label/([^/?#]+)"
+    test = (
+        ("https://dmmagazine.blogspot.com/search/label/D%26D", {
+            "range": "1-25",
+            "count": 25,
+            "keyword": {"label": "D&D"},
+        }),
+    )
+
+    def __init__(self, match):
+        BloggerExtractor.__init__(self, match)
+        self.label = text.unquote(match.group(3))
+
+    def posts(self, blog):
+        return self.api.blog_posts(blog["id"], self.label)
+
+    def metadata(self):
+        return {"label": self.label}
 
 
 class BloggerAPI():
