@@ -6,7 +6,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extractors for Gelbooru v0.2 sites"""
+"""Extractors for Gelbooru Beta 0.2 sites"""
 
 from . import booru
 from .. import text, util, exception
@@ -21,11 +21,18 @@ class GelbooruV02Extractor(booru.BooruExtractor):
 
     def __init__(self, match):
         booru.BooruExtractor.__init__(self, match)
+        self.api_key = self.config("api-key")
+        self.user_id = self.config("user-id")
+
         try:
             self.api_root = INSTANCES[self.category]["api_root"]
         except KeyError:
             self.api_root = self.root
 
+        if self.category == "realbooru":
+            self._file_url = self._file_url_realbooru
+            self._tags = self._tags_realbooru
+
     def _api_request(self, params):
         url = self.api_root + "/index.php?page=dapi&s=post&q=index"
         return ElementTree.fromstring(self.request(url, params=params).text)
@@ -56,60 +63,105 @@ class GelbooruV02Extractor(booru.BooruExtractor):
                 return
             params["pid"] += 1
 
+    def _pagination_html(self, params):
+        url = self.root + "/index.php"
+        params["pid"] = self.page_start * self.per_page
+
+        data = {}
+        while True:
+            num_ids = 0
+            page = self.request(url, params=params).text
+
+            for data["id"] in text.extract_iter(page, '" id="p', '"'):
+                num_ids += 1
+                for post in self._api_request(data):
+                    yield post.attrib
+
+            if num_ids < self.per_page:
+                return
+            params["pid"] += self.per_page
+
     @staticmethod
     def _prepare(post):
         post["date"] = text.parse_datetime(
             post["created_at"], "%a %b %d %H:%M:%S %z %Y")
 
-    def _extended_tags(self, post, page=None):
-        if not page:
-            url = "{}/index.php?page=post&s=view&id={}".format(
-                self.root, post["id"])
-            page = self.request(url).text
-        html = text.extract(page, '<ul id="tag-', '</ul>')[0]
-        if not html:
-            html = text.extract(page, '<ul class="tag-', '</ul>')[0]
-        if html:
-            tags = collections.defaultdict(list)
-            pattern = re.compile(
-                r"tag-type-([^\"' ]+).*?[?;]tags=([^\"'&]+)", re.S)
-            for tag_type, tag_name in pattern.findall(html):
-                tags[tag_type].append(text.unquote(tag_name))
-            for key, value in tags.items():
-                post["tags_" + key] = " ".join(value)
-        return page
+    def _html(self, post):
+        return self.request("{}/index.php?page=post&s=view&id={}".format(
+            self.root, post["id"])).text
 
-    def _notes(self, post, page=None):
-        if not page:
-            url = "{}/index.php?page=post&s=view&id={}".format(
-                self.root, post["id"])
-            page = self.request(url).text
-        notes = []
-        notes_data = text.extract(page, '<section id="notes"', '</section>')[0]
-        if not notes_data:
+    def _tags(self, post, page):
+        tag_container = (text.extr(page, '<ul id="tag-', '</ul>') or
+                         text.extr(page, '<ul class="tag-', '</ul>'))
+        if not tag_container:
             return
 
-        note_iter = text.extract_iter(notes_data, '<article', '</article>')
-        extr = text.extract
-        for note_data in note_iter:
-            note = {
-                "width": int(extr(note_data, 'data-width="', '"')[0]),
-                "height": int(extr(note_data, 'data-height="', '"')[0]),
-                "x": int(extr(note_data, 'data-x="', '"')[0]),
-                "y": int(extr(note_data, 'data-y="', '"')[0]),
-                "body": extr(note_data, 'data-body="', '"')[0],
-            }
-            notes.append(note)
+        tags = collections.defaultdict(list)
+        pattern = re.compile(
+            r"tag-type-([^\"' ]+).*?[?;]tags=([^\"'&]+)", re.S)
+        for tag_type, tag_name in pattern.findall(tag_container):
+            tags[tag_type].append(text.unquote(tag_name))
+        for key, value in tags.items():
+            post["tags_" + key] = " ".join(value)
 
-        post["notes"] = notes
+    def _notes(self, post, page):
+        note_container = text.extr(page, 'id="note-container"', "<img ")
+        if not note_container:
+            return
+
+        post["notes"] = notes = []
+        for note in note_container.split('class="note-box"')[1:]:
+            extr = text.extract_from(note)
+            notes.append({
+                "width" : int(extr("width:", "p")),
+                "height": int(extr("height:", "p")),
+                "y"     : int(extr("top:", "p")),
+                "x"     : int(extr("left:", "p")),
+                "id"    : int(extr('id="note-body-', '"')),
+                "body"  : text.unescape(text.remove_html(extr(">", "</div>"))),
+            })
+
+    def _file_url_realbooru(self, post):
+        url = post["file_url"]
+        md5 = post["md5"]
+        if md5 not in post["preview_url"] or url.count("/") == 5:
+            url = "{}/images/{}/{}/{}.{}".format(
+                self.root, md5[0:2], md5[2:4], md5, url.rpartition(".")[2])
+        return url
+
+    def _tags_realbooru(self, post, page):
+        tag_container = text.extr(page, 'id="tagLink"', '</div>')
+        tags = collections.defaultdict(list)
+        pattern = re.compile(
+            r'<a class="(?:tag-type-)?([^"]+).*?;tags=([^"&]+)')
+        for tag_type, tag_name in pattern.findall(tag_container):
+            tags[tag_type].append(text.unquote(tag_name))
+        for key, value in tags.items():
+            post["tags_" + key] = " ".join(value)
 
 
 INSTANCES = {
-    "realbooru": {"root": "https://realbooru.com"},
-    "rule34"   : {"root": "https://rule34.xxx",
-                  "api_root": " https://api.rule34.xxx"},
-    "safebooru": {"root": "https://safebooru.org"},
-    "tbib"     : {"root": "https://tbib.org"},
+    "realbooru": {
+        "root": "https://realbooru.com",
+        "pattern": r"realbooru\.com",
+    },
+    "rule34": {
+        "root": "https://rule34.xxx",
+        "pattern": r"rule34\.xxx",
+        "api_root": "https://api.rule34.xxx",
+    },
+    "safebooru": {
+        "root": "https://safebooru.org",
+        "pattern": r"safebooru\.org",
+    },
+    "tbib": {
+        "root": "https://tbib.org",
+        "pattern": r"tbib\.org",
+    },
+    "hypnohub": {
+        "root": "https://hypnohub.net",
+        "pattern": r"hypnohub\.net",
+    },
 }
 
 BASE_PATTERN = GelbooruV02Extractor.update(INSTANCES)
@@ -122,7 +174,8 @@ class GelbooruV02TagExtractor(GelbooruV02Extractor):
     pattern = BASE_PATTERN + r"/index\.php\?page=post&s=list&tags=([^&#]+)"
     test = (
         ("https://rule34.xxx/index.php?page=post&s=list&tags=danraku", {
-            "content": "622e80be3f496672c44aab5c47fbc6941c61bc79",
+            "content": ("5c6ae9ee13e6d4bc9cb8bdce224c84e67fbfa36c",
+                        "622e80be3f496672c44aab5c47fbc6941c61bc79"),
             "pattern": r"https?://.*rule34\.xxx/images/\d+/[0-9a-f]+\.jpg",
             "count": 2,
         }),
@@ -136,6 +189,9 @@ class GelbooruV02TagExtractor(GelbooruV02Extractor):
         ("https://tbib.org/index.php?page=post&s=list&tags=yuyaiyaui", {
             "count": ">= 120",
         }),
+        ("https://hypnohub.net/index.php?page=post&s=list&tags=gonoike_biwa", {
+            "url": "fe662b86d38c331fcac9c62af100167d404937dc",
+        }),
     )
 
     def __init__(self, match):
@@ -147,7 +203,7 @@ class GelbooruV02TagExtractor(GelbooruV02Extractor):
         return {"search_tags": self.tags}
 
     def posts(self):
-        return self._pagination({"tags" : self.tags})
+        return self._pagination({"tags": self.tags})
 
 
 class GelbooruV02PoolExtractor(GelbooruV02Extractor):
@@ -165,12 +221,21 @@ class GelbooruV02PoolExtractor(GelbooruV02Extractor):
         ("https://realbooru.com/index.php?page=pool&s=show&id=1", {
             "count": 3,
         }),
+        ("https://hypnohub.net/index.php?page=pool&s=show&id=61", {
+            "url": "d314826280073441a2da609f70ee814d1f4b9407",
+            "count": 3,
+        }),
     )
 
     def __init__(self, match):
         GelbooruV02Extractor.__init__(self, match)
         self.pool_id = match.group(match.lastindex)
-        self.post_ids = ()
+
+        if self.category == "rule34":
+            self.posts = self._posts_pages
+            self.per_page = 45
+        else:
+            self.post_ids = ()
 
     def skip(self, num):
         self.page_start += num
@@ -198,6 +263,13 @@ class GelbooruV02PoolExtractor(GelbooruV02Extractor):
             for post in self._api_request(params):
                 yield post.attrib
 
+    def _posts_pages(self):
+        return self._pagination_html({
+            "page": "pool",
+            "s"   : "show",
+            "id"  : self.pool_id,
+        })
+
 
 class GelbooruV02FavoriteExtractor(GelbooruV02Extractor):
     subcategory = "favorite"
@@ -213,11 +285,14 @@ class GelbooruV02FavoriteExtractor(GelbooruV02Extractor):
             "count": 2,
         }),
         ("https://realbooru.com/index.php?page=favorites&s=view&id=274", {
-            "count": 4,
+            "count": 2,
         }),
         ("https://tbib.org/index.php?page=favorites&s=view&id=7881", {
             "count": 3,
         }),
+        ("https://hypnohub.net/index.php?page=favorites&s=view&id=43546", {
+            "count": 3,
+        }),
     )
 
     def __init__(self, match):
@@ -228,27 +303,11 @@ class GelbooruV02FavoriteExtractor(GelbooruV02Extractor):
         return {"favorite_id": text.parse_int(self.favorite_id)}
 
     def posts(self):
-        url = self.root + "/index.php"
-        params = {
+        return self._pagination_html({
             "page": "favorites",
             "s"   : "view",
             "id"  : self.favorite_id,
-            "pid" : self.page_start * self.per_page,
-        }
-
-        data = {}
-        while True:
-            num_ids = 0
-            page = self.request(url, params=params).text
-
-            for data["id"] in text.extract_iter(page, '" id="p', '"'):
-                num_ids += 1
-                for post in self._api_request(data):
-                    yield post.attrib
-
-            if num_ids < self.per_page:
-                return
-            params["pid"] += self.per_page
+        })
 
 
 class GelbooruV02PostExtractor(GelbooruV02Extractor):
@@ -256,15 +315,81 @@ class GelbooruV02PostExtractor(GelbooruV02Extractor):
     archive_fmt = "{id}"
     pattern = BASE_PATTERN + r"/index\.php\?page=post&s=view&id=(\d+)"
     test = (
-        ("https://rule34.xxx/index.php?page=post&s=view&id=1995545", {
-            "content": "97e4bbf86c3860be18de384d02d544251afe1d45",
-            "options": (("tags", True),),
+        ("https://rule34.xxx/index.php?page=post&s=view&id=863", {
+            "pattern": r"https://api-cdn\.rule34\.xxx/images"
+                       r"/1/6aafbdb3e22f3f3b412ea2cf53321317a37063f3\.jpg",
+            "content": ("a43f418aa350039af0d11cae501396a33bbe2201",
+                        "67b516295950867e1c1ab6bc13b35d3b762ed2a3"),
+            "options": (("tags", True), ("notes", True)),
             "keyword": {
-                "tags_artist": "danraku",
-                "tags_character": "kashima_(kantai_collection)",
-                "tags_copyright": "kantai_collection",
+                "tags_artist": "reverse_noise yamu_(reverse_noise)",
+                "tags_character": "hong_meiling",
+                "tags_copyright": "touhou",
                 "tags_general": str,
-                "tags_metadata": str,
+                "tags_metadata": "censored translated",
+                "notes": [
+                    {
+                        "body": "It feels angry, I'm losing myself... "
+                                "It won't calm down!",
+                        "height": 65,
+                        "id": 93586,
+                        "width": 116,
+                        "x": 22,
+                        "y": 333,
+                    },
+                    {
+                        "body": "REPUTATION OF RAGE",
+                        "height": 272,
+                        "id": 93587,
+                        "width": 199,
+                        "x": 78,
+                        "y": 442,
+                    },
+                ],
+
+            },
+        }),
+        ("https://hypnohub.net/index.php?page=post&s=view&id=1439", {
+            "pattern": r"https://hypnohub\.net/images"
+                       r"/90/24/90245c3c5250c2a8173255d3923a010b\.jpg",
+            "content": "5987c5d2354f22e5fa9b7ee7ce4a6f7beb8b2b71",
+            "options": (("tags", True), ("notes", True)),
+            "keyword": {
+                "tags_artist": "brokenteapot",
+                "tags_character": "hsien-ko",
+                "tags_copyright": "capcom darkstalkers",
+                "tags_general": str,
+                "tags_metadata": "dialogue text translated",
+                "notes": [
+                    {
+                        "body": "Master Master Master "
+                                "Master Master Master",
+                        "height": 83,
+                        "id": 10577,
+                        "width": 129,
+                        "x": 259,
+                        "y": 20,
+                    },
+                    {
+                        "body": "Response Response Response "
+                                "Response Response Response",
+                        "height": 86,
+                        "id": 10578,
+                        "width": 125,
+                        "x": 126,
+                        "y": 20,
+                    },
+                    {
+                        "body": "Obedience Obedience Obedience "
+                                "Obedience Obedience Obedience",
+                        "height": 80,
+                        "id": 10579,
+                        "width": 98,
+                        "x": 20,
+                        "y": 20,
+                    },
+                ],
+
             },
         }),
         ("https://safebooru.org/index.php?page=post&s=view&id=1169132", {
@@ -279,8 +404,16 @@ class GelbooruV02PostExtractor(GelbooruV02Extractor):
             },
         }),
         ("https://realbooru.com/index.php?page=post&s=view&id=668483", {
-            "url": "2421b5b0e15d5e20f9067090a8b0fd4114d3e7d9",
+            "pattern": r"https://realbooru\.com/images/dc/b5"
+                       r"/dcb5c0ce9ec0bf74a6930608985f4719\.jpeg",
             "content": "7f5873ce3b6cd295ea2e81fcb49583098ea9c8da",
+            "options": (("tags", True),),
+            "keyword": {
+                "tags_general": "1girl blonde blonde_hair blue_eyes cute "
+                                "female female_only looking_at_viewer smile "
+                                "solo solo_female teeth",
+                "tags_model": "jennifer_lawrence",
+            },
         }),
         ("https://tbib.org/index.php?page=post&s=view&id=9233957", {
             "url": "5a6ebe07bfff8e6d27f7c30b5480f27abcb577d2",
