@@ -1,25 +1,26 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2020-2021 Mike Fährmann
+# Copyright 2020-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extractors for https://hiperdex.com/"""
+"""Extractors for https://1sthiperdex.com/"""
 
 from .common import ChapterExtractor, MangaExtractor
 from .. import text
 from ..cache import memcache
 import re
 
-BASE_PATTERN = r"(?:https?://)?(?:www\.)?hiperdex\d?\.(?:com|net|info)"
+BASE_PATTERN = (r"((?:https?://)?(?:www\.)?"
+                r"(?:1st)?hiperdex\d?\.(?:com|net|info))")
 
 
 class HiperdexBase():
     """Base class for hiperdex extractors"""
     category = "hiperdex"
-    root = "https://hiperdex2.com"
+    root = "https://1sthiperdex.com"
 
     @memcache(keyarg=1)
     def manga_data(self, manga, page=None):
@@ -52,6 +53,8 @@ class HiperdexBase():
         }
 
     def chapter_data(self, chapter):
+        if chapter.startswith("chapter-"):
+            chapter = chapter[8:]
         chapter, _, minor = chapter.partition("-")
         data = {
             "chapter"      : text.parse_int(chapter),
@@ -62,12 +65,13 @@ class HiperdexBase():
 
 
 class HiperdexChapterExtractor(HiperdexBase, ChapterExtractor):
-    """Extractor for manga chapters from hiperdex.com"""
+    """Extractor for manga chapters from 1sthiperdex.com"""
     pattern = BASE_PATTERN + r"(/manga/([^/?#]+)/([^/?#]+))"
     test = (
-        ("https://hiperdex2.com/manga/domestic-na-kanojo/154-5/", {
-            "pattern": r"https://hiperdex\d?.(com|net|info)/wp-content/uploads"
-                       r"/WP-manga/data/manga_\w+/[0-9a-f]{32}/\d+\.webp",
+        ("https://1sthiperdex.com/manga/domestic-na-kanojo/154-5/", {
+            "pattern": r"https://(1st)?hiperdex\d?.(com|net|info)"
+                       r"/wp-content/uploads/WP-manga/data"
+                       r"/manga_\w+/[0-9a-f]{32}/\d+\.webp",
             "count": 9,
             "keyword": {
                 "artist" : "Sasuga Kei",
@@ -82,12 +86,15 @@ class HiperdexChapterExtractor(HiperdexBase, ChapterExtractor):
                 "type"   : "Manga",
             },
         }),
+        ("https://hiperdex.com/manga/domestic-na-kanojo/154-5/"),
+        ("https://hiperdex2.com/manga/domestic-na-kanojo/154-5/"),
         ("https://hiperdex.net/manga/domestic-na-kanojo/154-5/"),
         ("https://hiperdex.info/manga/domestic-na-kanojo/154-5/"),
     )
 
     def __init__(self, match):
-        path, self.manga, self.chapter = match.groups()
+        root, path, self.manga, self.chapter = match.groups()
+        self.root = text.ensure_http_scheme(root)
         ChapterExtractor.__init__(self, match, self.root + path + "/")
 
     def metadata(self, _):
@@ -102,11 +109,11 @@ class HiperdexChapterExtractor(HiperdexBase, ChapterExtractor):
 
 
 class HiperdexMangaExtractor(HiperdexBase, MangaExtractor):
-    """Extractor for manga from hiperdex.com"""
+    """Extractor for manga from 1sthiperdex.com"""
     chapterclass = HiperdexChapterExtractor
     pattern = BASE_PATTERN + r"(/manga/([^/?#]+))/?$"
     test = (
-        ("https://hiperdex2.com/manga/youre-not-that-special/", {
+        ("https://1sthiperdex.com/manga/youre-not-that-special/", {
             "count": 51,
             "pattern": HiperdexChapterExtractor.pattern,
             "keyword": {
@@ -123,27 +130,34 @@ class HiperdexMangaExtractor(HiperdexBase, MangaExtractor):
                 "type"   : "Manhwa",
             },
         }),
+        ("https://hiperdex.com/manga/youre-not-that-special/"),
+        ("https://hiperdex2.com/manga/youre-not-that-special/"),
         ("https://hiperdex.net/manga/youre-not-that-special/"),
         ("https://hiperdex.info/manga/youre-not-that-special/"),
     )
 
     def __init__(self, match):
-        path, self.manga = match.groups()
+        root, path, self.manga = match.groups()
+        self.root = text.ensure_http_scheme(root)
         MangaExtractor.__init__(self, match, self.root + path + "/")
 
     def chapters(self, page):
         self.manga_data(self.manga, page)
         results = []
 
-        shortlink = text.extract(page, "rel='shortlink' href='", "'")[0]
+        shortlink = text.extr(page, "rel='shortlink' href='", "'")
         data = {
-            "action": "manga_get_chapters",
-            "manga" : shortlink.rpartition("=")[2],
+            "action"   : "manga_get_reading_nav",
+            "manga"    : shortlink.rpartition("=")[2],
+            "chapter"  : "",
+            "volume_id": "",
+            "style"    : "list",
+            "type"     : "manga",
         }
         url = self.root + "/wp-admin/admin-ajax.php"
         page = self.request(url, method="POST", data=data).text
 
-        for url in text.extract_iter(page, 'href="', '"', 320):
+        for url in text.extract_iter(page, 'data-redirect="', '"'):
             chapter = url.rpartition("/")[2]
             results.append((url, self.chapter_data(chapter)))
 
@@ -156,10 +170,11 @@ class HiperdexArtistExtractor(HiperdexBase, MangaExtractor):
     categorytransfer = False
     chapterclass = HiperdexMangaExtractor
     reverse = False
-    pattern = BASE_PATTERN + r"(/manga-a(?:rtist|uthor)/([^/?#]+))"
+    pattern = BASE_PATTERN + r"(/manga-a(?:rtist|uthor)/(?:[^/?#]+))"
     test = (
-        ("https://hiperdex2.com/manga-artist/beck-ho-an/"),
+        ("https://1sthiperdex.com/manga-artist/beck-ho-an/"),
         ("https://hiperdex.net/manga-artist/beck-ho-an/"),
+        ("https://hiperdex2.com/manga-artist/beck-ho-an/"),
         ("https://hiperdex.info/manga-artist/beck-ho-an/"),
         ("https://hiperdex.com/manga-author/viagra/", {
             "pattern": HiperdexMangaExtractor.pattern,
@@ -168,11 +183,12 @@ class HiperdexArtistExtractor(HiperdexBase, MangaExtractor):
     )
 
     def __init__(self, match):
-        MangaExtractor.__init__(self, match, self.root + match.group(1) + "/")
+        self.root = text.ensure_http_scheme(match.group(1))
+        MangaExtractor.__init__(self, match, self.root + match.group(2) + "/")
 
     def chapters(self, page):
         results = []
         for info in text.extract_iter(page, 'id="manga-item-', '<img'):
-            url = text.extract(info, 'href="', '"')[0]
+            url = text.extr(info, 'href="', '"')
             results.append((url, {}))
         return results
