@@ -187,12 +187,19 @@ class ImagevenueImageExtractor(ImagehostImageExtractor):
 class ImagetwistImageExtractor(ImagehostImageExtractor):
     """Extractor for single images from imagetwist.com"""
     category = "imagetwist"
-    pattern = r"(?:https?://)?((?:www\.)?imagetwist\.com/([a-z0-9]{12}))"
-    test = ("https://imagetwist.com/f1i2s4vhvbrq/test.png", {
-        "url": "8d5e168c0bee30211f821c6f3b2116e419d42671",
-        "keyword": "d1060a4c2e3b73b83044e20681712c0ffdd6cfef",
-        "content": "0c8768055e4e20e7c7259608b67799171b691140",
-    })
+    pattern = (r"(?:https?://)?((?:www\.|phun\.)?"
+               r"image(?:twist|haha)\.com/([a-z0-9]{12}))")
+    test = (
+        ("https://imagetwist.com/f1i2s4vhvbrq/test.png", {
+            "url": "8d5e168c0bee30211f821c6f3b2116e419d42671",
+            "keyword": "d1060a4c2e3b73b83044e20681712c0ffdd6cfef",
+            "content": "0c8768055e4e20e7c7259608b67799171b691140",
+        }),
+        ("https://www.imagetwist.com/f1i2s4vhvbrq/test.png"),
+        ("https://phun.imagetwist.com/f1i2s4vhvbrq/test.png"),
+        ("https://imagehaha.com/f1i2s4vhvbrq/test.png"),
+        ("https://www.imagehaha.com/f1i2s4vhvbrq/test.png"),
+    )
 
     @property
     @memcache(maxage=3*3600)
@@ -200,7 +207,7 @@ class ImagetwistImageExtractor(ImagehostImageExtractor):
         return self.request(self.page_url).cookies
 
     def get_info(self, page):
-        url     , pos = text.extract(page, 'center;"><img src="', '"')
+        url     , pos = text.extract(page, '<img src="', '"')
         filename, pos = text.extract(page, ' alt="', '"', pos)
         return url, filename
 
@@ -242,6 +249,25 @@ class PixhostImageExtractor(ImagehostImageExtractor):
         return url, filename
 
 
+class PixhostGalleryExtractor(ImagehostImageExtractor):
+    """Extractor for image galleries from pixhost.to"""
+    category = "pixhost"
+    subcategory = "gallery"
+    pattern = (r"(?:https?://)?((?:www\.)?pixhost\.(?:to|org)"
+               r"/gallery/([^/?#]+))")
+    test = ("https://pixhost.to/gallery/jSMFq", {
+        "pattern": PixhostImageExtractor.pattern,
+        "count": 3,
+    })
+
+    def items(self):
+        page = text.extr(self.request(
+            self.page_url).text, 'class="images"', "</div>")
+        data = {"_extractor": PixhostImageExtractor}
+        for url in text.extract_iter(page, '<a href="', '"'):
+            yield Message.Queue, url, data
+
+
 class PostimgImageExtractor(ImagehostImageExtractor):
     """Extractor for single images from postimages.org"""
     category = "postimg"
