@@ -9,9 +9,8 @@
 """Extractors for https://imgbb.com/"""
 
 from .common import Extractor, Message
-from .. import text, exception
+from .. import text, util, exception
 from ..cache import cache
-import json
 
 
 class ImgbbExtractor(Extractor):
@@ -43,7 +42,6 @@ class ImgbbExtractor(Extractor):
         data = self.metadata(page)
         first = True
 
-        yield Message.Version, 1
         for img in self.images(page):
             image = {
                 "id"       : img["url_viewer"].rpartition("/")[2],
@@ -72,7 +70,7 @@ class ImgbbExtractor(Extractor):
 
         url = self.root + "/login"
         page = self.request(url).text
-        token = text.extract(page, 'PF.obj.config.auth_token="', '"')[0]
+        token = text.extr(page, 'PF.obj.config.auth_token="', '"')
 
         headers = {"Referer": url}
         data = {
@@ -99,7 +97,7 @@ class ImgbbExtractor(Extractor):
 
         while True:
             for img in text.extract_iter(page, "data-object='", "'"):
-                yield json.loads(text.unquote(img))
+                yield util.json_loads(text.unquote(img))
             if data:
                 if params["seek"] == data["seekEnd"]:
                     return
@@ -155,7 +153,7 @@ class ImgbbAlbumExtractor(ImgbbExtractor):
         }
 
     def images(self, page):
-        url = text.extract(page, '"og:url" content="', '"')[0]
+        url = text.extr(page, '"og:url" content="', '"')
         album_id = url.rpartition("/")[2].partition("?")[0]
 
         return self._pagination(page, "https://ibb.co/json", {
@@ -170,7 +168,7 @@ class ImgbbAlbumExtractor(ImgbbExtractor):
 class ImgbbUserExtractor(ImgbbExtractor):
     """Extractor for user profiles in imgbb.com"""
     subcategory = "user"
-    pattern = r"(?:https?://)?([^.]+)\.imgbb\.com/?(?:\?([^#]+))?$"
+    pattern = r"(?:https?://)?([\w-]+)\.imgbb\.com/?(?:\?([^#]+))?$"
     test = ("https://folkie.imgbb.com", {
         "range": "1-80",
         "pattern": r"https?://i\.ibb\.co/\w+/[^/?#]+",
@@ -186,7 +184,7 @@ class ImgbbUserExtractor(ImgbbExtractor):
         return {"user": self.user}
 
     def images(self, page):
-        user = text.extract(page, '.obj.resource={"id":"', '"')[0]
+        user = text.extr(page, '.obj.resource={"id":"', '"')
         return self._pagination(page, self.page_url + "json", {
             "from"      : "user",
             "userid"    : user,
@@ -230,6 +228,5 @@ class ImgbbImageExtractor(ImgbbExtractor):
         }
         image["extension"] = text.ext_from_url(image["url"])
 
-        yield Message.Version, 1
         yield Message.Directory, image
         yield Message.Url, image["url"], image
