@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2019-2021 Mike Fährmann
+# Copyright 2019-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -31,6 +31,8 @@ class MastodonExtractor(BaseExtractor):
     def items(self):
         for status in self.statuses():
 
+            if self._check_moved:
+                self._check_moved(status["account"])
             if not self.reblogs and status["reblog"]:
                 self.log.debug("Skipping %s (reblog)", status["id"])
                 continue
@@ -42,12 +44,17 @@ class MastodonExtractor(BaseExtractor):
             del status["media_attachments"]
 
             status["instance"] = self.instance
+            acct = status["account"]["acct"]
+            status["instance_remote"] = \
+                acct.rpartition("@")[2] if "@" in acct else None
+
+            status["count"] = len(attachments)
             status["tags"] = [tag["name"] for tag in status["tags"]]
             status["date"] = text.parse_datetime(
                 status["created_at"][:19], "%Y-%m-%dT%H:%M:%S")
 
             yield Message.Directory, status
-            for media in attachments:
+            for status["num"], media in enumerate(attachments, 1):
                 status["media"] = media
                 url = media["url"]
                 yield Message.Url, url, text.nameext_from_url(url, status)
@@ -56,16 +63,24 @@ class MastodonExtractor(BaseExtractor):
         """Return an iterable containing all relevant Status objects"""
         return ()
 
+    def _check_moved(self, account):
+        self._check_moved = None
+        if "moved" in account:
+            self.log.warning("Account '%s' moved to '%s'",
+                             account["acct"], account["moved"]["acct"])
+
 
 INSTANCES = {
     "mastodon.social": {
         "root"         : "https://mastodon.social",
+        "pattern"      : r"mastodon\.social",
         "access-token" : "Y06R36SMvuXXN5_wiPKFAEFiQaMSQg0o_hGgc86Jj48",
         "client-id"    : "dBSHdpsnOUZgxOnjKSQrWEPakO3ctM7HmsyoOd4FcRo",
         "client-secret": "DdrODTHs_XoeOsNVXnILTMabtdpWrWOAtrmw91wU1zI",
     },
     "pawoo": {
         "root"         : "https://pawoo.net",
+        "pattern"      : r"pawoo\.net",
         "access-token" : "c12c9d275050bce0dc92169a28db09d7"
                          "0d62d0a75a8525953098c167eacd3668",
         "client-id"    : "978a25f843ec01e53d09be2c290cd75c"
@@ -75,13 +90,14 @@ INSTANCES = {
     },
     "baraag": {
         "root"         : "https://baraag.net",
+        "pattern"      : r"baraag\.net",
         "access-token" : "53P1Mdigf4EJMH-RmeFOOSM9gdSDztmrAYFgabOKKE0",
         "client-id"    : "czxx2qilLElYHQ_sm-lO8yXuGwOHxLX9RYYaD0-nq1o",
         "client-secret": "haMaFdMBgK_-BIxufakmI2gFgkYjqmgXGEO2tB-R2xY",
     }
 }
 
-BASE_PATTERN = MastodonExtractor.update(INSTANCES)
+BASE_PATTERN = MastodonExtractor.update(INSTANCES) + "(?:/web)?"
 
 
 class MastodonUserExtractor(MastodonExtractor):
@@ -100,9 +116,16 @@ class MastodonUserExtractor(MastodonExtractor):
             "count": 60,
         }),
         ("https://baraag.net/@pumpkinnsfw"),
+        ("https://mastodon.social/@yoru_nine@pawoo.net", {
+            "pattern": r"https://mastodon\.social/media_proxy/\d+/original",
+            "range": "1-10",
+            "count": 10,
+        }),
         ("https://mastodon.social/@id:10843"),
         ("https://mastodon.social/users/id:10843"),
         ("https://mastodon.social/users/jk"),
+        ("https://mastodon.social/users/yoru_nine@pawoo.net"),
+        ("https://mastodon.social/web/@jk"),
     )
 
     def statuses(self):
@@ -115,6 +138,20 @@ class MastodonUserExtractor(MastodonExtractor):
         )
 
 
+class MastodonBookmarkExtractor(MastodonExtractor):
+    """Extractor for mastodon bookmarks"""
+    subcategory = "bookmark"
+    pattern = BASE_PATTERN + r"/bookmarks"
+    test = (
+        ("https://mastodon.social/bookmarks"),
+        ("https://pawoo.net/bookmarks"),
+        ("https://baraag.net/bookmarks"),
+    )
+
+    def statuses(self):
+        return MastodonAPI(self).account_bookmarks()
+
+
 class MastodonFollowingExtractor(MastodonExtractor):
     """Extractor for followed mastodon users"""
     subcategory = "following"
@@ -145,6 +182,10 @@ class MastodonStatusExtractor(MastodonExtractor):
     test = (
         ("https://mastodon.social/@jk/103794036899778366", {
             "count": 4,
+            "keyword": {
+                "count": 4,
+                "num": int,
+            },
         }),
         ("https://pawoo.net/@yoru_nine/105038878897832922", {
             "content": "b52e807f8ab548d6f896b09218ece01eba83987a",
@@ -176,23 +217,31 @@ class MastodonAPI():
             try:
                 access_token = INSTANCES[extractor.category]["access-token"]
             except (KeyError, TypeError):
-                raise exception.StopExtraction(
-                    "Missing access token.\n"
-                    "Run 'gallery-dl oauth:mastodon:%s' to obtain one.",
-                    extractor.instance)
-
-        self.headers = {"Authorization": "Bearer " + access_token}
+                pass
+        if access_token:
+            self.headers = {"Authorization": "Bearer " + access_token}
+        else:
+            self.headers = None
 
     def account_id_by_username(self, username):
         if username.startswith("id:"):
             return username[3:]
 
-        handle = "@{}@{}".format(username, self.extractor.instance)
+        if "@" in username:
+            handle = "@" + username
+        else:
+            handle = "@{}@{}".format(username, self.extractor.instance)
+
         for account in self.account_search(handle, 1):
-            if account["username"] == username:
+            if account["acct"] == username:
+                self.extractor._check_moved(account)
                 return account["id"]
         raise exception.NotFoundError("account")
 
+    def account_bookmarks(self):
+        endpoint = "/v1/bookmarks"
+        return self._pagination(endpoint, None)
+
     def account_following(self, account_id):
         endpoint = "/v1/accounts/{}/following".format(account_id)
         return self._pagination(endpoint, None)
@@ -229,6 +278,11 @@ class MastodonAPI():
 
             if code < 400:
                 return response
+            if code == 401:
+                raise exception.StopExtraction(
+                    "Invalid or missing access token.\n"
+                    "Run 'gallery-dl oauth:mastodon:%s' to obtain one.",
+                    self.extractor.instance)
             if code == 404:
                 raise exception.NotFoundError()
             if code == 429:
