@@ -59,10 +59,7 @@ class NitterExtractor(BaseExtractor):
 
                     if url[0] == "/":
                         url = self.root + url
-                    file = {
-                        "url": url,
-                        "_http_retry_codes": (404,),
-                    }
+                    file = {"url": url, "_http_retry": _retry_on_404}
                     file["filename"], _, file["extension"] = \
                         name.rpartition(".")
                     append(file)
@@ -220,10 +217,6 @@ BASE_PATTERN = NitterExtractor.update({
         "root": "https://nitter.lacontrevoie.fr",
         "pattern": r"nitter\.lacontrevoie\.fr",
     },
-    "nitter.pussthecat.org": {
-        "root": "https://nitter.pussthecat.org",
-        "pattern": r"nitter\.pussthecat\.org",
-    },
     "nitter.1d4.us": {
         "root": "https://nitter.1d4.us",
         "pattern": r"nitter\.1d4\.us",
@@ -283,13 +276,12 @@ class NitterTweetsExtractor(NitterExtractor):
                 },
             },
         }),
-        ("https://nitter.pussthecat.org/i/user/2976459548", {
-            "url": "c740a2683db2c8ed2f350afc0494475c4444025b",
-            "pattern": r"https://nitter.pussthecat\.org/pic/orig"
+        ("https://nitter.lacontrevoie.fr/supernaturepics", {
+            "url": "54f4b55f2099dcc248f3fb7bfacf1349e08d8e2d",
+            "pattern": r"https://nitter\.lacontrevoie\.fr/pic/orig"
                        r"/media%2FCGMNYZvW0AIVoom\.jpg",
             "range": "1",
         }),
-        ("https://nitter.lacontrevoie.fr/supernaturepics"),
         ("https://nitter.1d4.us/supernaturepics"),
         ("https://nitter.kavin.rocks/id:2976459548"),
         ("https://nitter.unixfox.eu/supernaturepics"),
@@ -309,7 +301,6 @@ class NitterRepliesExtractor(NitterExtractor):
             "range": "1-20",
         }),
         ("https://nitter.lacontrevoie.fr/supernaturepics/with_replies"),
-        ("https://nitter.pussthecat.org/supernaturepics/with_replies"),
         ("https://nitter.1d4.us/supernaturepics/with_replies"),
         ("https://nitter.kavin.rocks/id:2976459548/with_replies"),
         ("https://nitter.unixfox.eu/i/user/2976459548/with_replies"),
@@ -334,7 +325,6 @@ class NitterMediaExtractor(NitterExtractor):
             "range": "1-20",
         }),
         ("https://nitter.lacontrevoie.fr/supernaturepics/media"),
-        ("https://nitter.pussthecat.org/supernaturepics/media"),
         ("https://nitter.1d4.us/supernaturepics/media"),
         ("https://nitter.unixfox.eu/i/user/2976459548/media"),
     )
@@ -353,7 +343,6 @@ class NitterSearchExtractor(NitterExtractor):
             "range": "1-20",
         }),
         ("https://nitter.lacontrevoie.fr/supernaturepics/search"),
-        ("https://nitter.pussthecat.org/supernaturepics/search"),
         ("https://nitter.1d4.us/supernaturepics/search"),
         ("https://nitter.kavin.rocks/id:2976459548/search"),
         ("https://nitter.unixfox.eu/i/user/2976459548/search"),
@@ -375,7 +364,7 @@ class NitterTweetExtractor(NitterExtractor):
             "url": "3f2b64e175bf284aa672c3bb53ed275e470b919a",
             "content": "ab05e1d8d21f8d43496df284d31e8b362cd3bcab",
             "keyword": {
-                "comments": 16,
+                "comments": 19,
                 "content": "Big Wedeene River, Canada",
                 "count": 1,
                 "date": "dt:2015-05-29 17:40:00",
@@ -399,9 +388,9 @@ class NitterTweetExtractor(NitterExtractor):
             "url": "9c51b3a4a1114535eb9b168bba97ad95db0d59ff",
         }),
         # video
-        ("https://nitter.pussthecat.org/i/status/1065692031626829824", {
-            "pattern": r"ytdl:https://nitter.pussthecat.org/video"
-                       r"/B875137EDC8FF/https%3A%2F%2Fvideo.twimg.com%2F"
+        ("https://nitter.lacontrevoie.fr/i/status/1065692031626829824", {
+            "pattern": r"ytdl:https://nitter\.lacontrevoie\.fr/video"
+                       r"/[0-9A-F]{10,}/https%3A%2F%2Fvideo.twimg.com%2F"
                        r"ext_tw_video%2F1065691868439007232%2Fpu%2Fpl%2F"
                        r"nv8hUQC1R0SjhzcZ.m3u8%3Ftag%3D5",
             "keyword": {
@@ -446,7 +435,7 @@ class NitterTweetExtractor(NitterExtractor):
             "count": 0,
         }),
         # "Misleading" content
-        ("https://nitter.pussthecat.org/i/status/1486373748911575046", {
+        ("https://nitter.lacontrevoie.fr/i/status/1486373748911575046", {
             "count": 4,
         }),
         # age-restricted (#2354)
@@ -468,3 +457,7 @@ class NitterTweetExtractor(NitterExtractor):
             quoted["user"] = tweet["user"]
             return (tweet, quoted)
         return (tweet,)
+
+
+def _retry_on_404(response):
+    return response.status_code == 404
