@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2014-2022 Mike Fährmann
+# Copyright 2014-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -37,7 +37,7 @@ class PixivExtractor(Extractor):
             transform_tags = None
         elif tags == "translated":
             def transform_tags(work):
-                work["tags"] = list(set(
+                work["tags"] = list(dict.fromkeys(
                     tag["translated_name"] or tag["name"]
                     for tag in work["tags"]))
         else:
@@ -45,7 +45,8 @@ class PixivExtractor(Extractor):
                 work["tags"] = [tag["name"] for tag in work["tags"]]
 
         ratings = {0: "General", 1: "R-18", 2: "R-18G"}
-        userdata = self.config("metadata")
+        meta_user = self.config("metadata")
+        meta_bookmark = self.config("metadata-bookmark")
         metadata = self.metadata()
 
         works = self.works()
@@ -61,8 +62,12 @@ class PixivExtractor(Extractor):
             del work["image_urls"]
             del work["meta_pages"]
 
-            if userdata:
+            if meta_user:
                 work.update(self.api.user_detail(work["user"]["id"]))
+            if meta_bookmark and work["is_bookmarked"]:
+                detail = self.api.illust_bookmark_detail(work["id"])
+                work["tags_bookmark"] = [tag["name"] for tag in detail["tags"]
+                                         if tag["is_registered"]]
             if transform_tags:
                 transform_tags(work)
             work["num"] = 0
@@ -88,19 +93,31 @@ class PixivExtractor(Extractor):
                 url = ugoira["zip_urls"]["medium"].replace(
                     "_ugoira600x600", "_ugoira1920x1080")
                 work["frames"] = ugoira["frames"]
+                work["date_url"] = self._date_from_url(url)
                 work["_http_adjust_extension"] = False
                 yield Message.Url, url, text.nameext_from_url(url, work)
 
             elif work["page_count"] == 1:
                 url = meta_single_page["original_image_url"]
+                work["date_url"] = self._date_from_url(url)
                 yield Message.Url, url, text.nameext_from_url(url, work)
 
             else:
                 for work["num"], img in enumerate(meta_pages):
                     url = img["image_urls"]["original"]
+                    work["date_url"] = self._date_from_url(url)
                     work["suffix"] = "_p{:02}".format(work["num"])
                     yield Message.Url, url, text.nameext_from_url(url, work)
 
+    @staticmethod
+    def _date_from_url(url, offset=timedelta(hours=9)):
+        try:
+            _, _, _, _, _, y, m, d, H, M, S, _ = url.split("/")
+            return datetime(
+                int(y), int(m), int(d), int(H), int(M), int(S)) - offset
+        except Exception:
+            return None
+
     @staticmethod
     def _make_work(kind, url, user):
         p = url.split("/")
@@ -309,6 +326,10 @@ class PixivWorkExtractor(PixivExtractor):
         ("https://www.pixiv.net/artworks/966412", {
             "url": "90c1715b07b0d1aad300bce256a0bc71f42540ba",
             "content": "69a8edfb717400d1c2e146ab2b30d2c235440c5a",
+            "keyword": {
+                "date"    : "dt:2008-06-12 15:29:13",
+                "date_url": "dt:2008-06-12 15:29:13",
+            },
         }),
         (("http://www.pixiv.net/member_illust.php"
           "?mode=medium&illust_id=966411"), {
@@ -318,7 +339,11 @@ class PixivWorkExtractor(PixivExtractor):
         (("https://www.pixiv.net/member_illust.php"
           "?mode=medium&illust_id=66806629"), {
             "url": "7267695a985c4db8759bebcf8d21dbdd2d2317ef",
-            "keywords": {"frames": list},
+            "keyword": {
+                "frames"  : list,
+                "date"    : "dt:2018-01-14 15:06:08",
+                "date_url": "dt:2018-01-15 04:24:48",
+            },
         }),
         # related works (#1237)
         ("https://www.pixiv.net/artworks/966412", {
@@ -378,6 +403,8 @@ class PixivFavoriteExtractor(PixivExtractor):
         # own bookmarks
         ("https://www.pixiv.net/bookmark.php", {
             "url": "90c1715b07b0d1aad300bce256a0bc71f42540ba",
+            "keyword": {"tags_bookmark": ["47", "hitman"]},
+            "options": (("metadata-bookmark", True),),
         }),
         # own bookmarks with tag (#596)
         ("https://www.pixiv.net/bookmark.php?tag=foobar", {
@@ -860,6 +887,11 @@ class PixivAppAPI():
         params = {"illust_id": illust_id}
         return self._call("/v1/illust/detail", params)["illust"]
 
+    def illust_bookmark_detail(self, illust_id):
+        params = {"illust_id": illust_id}
+        return self._call(
+            "/v2/illust/bookmark/detail", params)["bookmark_detail"]
+
     def illust_follow(self, restrict="all"):
         params = {"restrict": restrict}
         return self._pagination("/v2/illust/follow", params)
@@ -880,9 +912,16 @@ class PixivAppAPI():
         return self._pagination("/v1/search/illust", params)
 
     def user_bookmarks_illust(self, user_id, tag=None, restrict="public"):
+        """Return illusts bookmarked by a user"""
         params = {"user_id": user_id, "tag": tag, "restrict": restrict}
         return self._pagination("/v1/user/bookmarks/illust", params)
 
+    def user_bookmark_tags_illust(self, user_id, restrict="public"):
+        """Return bookmark tags defined by a user"""
+        params = {"user_id": user_id, "restrict": restrict}
+        return self._pagination(
+            "/v1/user/bookmark-tags/illust", params, "bookmark_tags")
+
     @memcache(keyarg=1)
     def user_detail(self, user_id):
         params = {"user_id": user_id}
