@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2020-2022 Mike Fährmann
+# Copyright 2020-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,9 +9,8 @@
 """Extractors for https://www.subscribestar.com/"""
 
 from .common import Extractor, Message
-from .. import text, exception
+from .. import text, util, exception
 from ..cache import cache
-import json
 
 BASE_PATTERN = r"(?:https?://)?(?:www\.)?subscribestar\.(com|adult)"
 
@@ -89,23 +88,23 @@ class SubscribestarExtractor(Extractor):
     def _media_from_post(html):
         media = []
 
-        gallery = text.extract(html, 'data-gallery="', '"')[0]
+        gallery = text.extr(html, 'data-gallery="', '"')
         if gallery:
             media.extend(
-                item for item in json.loads(text.unescape(gallery))
+                item for item in util.json_loads(text.unescape(gallery))
                 if "/previews/" not in item["url"]
             )
 
-        attachments = text.extract(
-            html, 'class="uploads-docs"', 'data-role="post-edit_form"')[0]
+        attachments = text.extr(
+            html, 'class="uploads-docs"', 'data-role="post-edit_form"')
         if attachments:
             for att in attachments.split('class="doc_preview"')[1:]:
                 media.append({
-                    "id"  : text.parse_int(text.extract(
-                        att, 'data-upload-id="', '"')[0]),
-                    "name": text.unescape(text.extract(
-                        att, 'doc_preview-title">', '<')[0] or ""),
-                    "url" : text.unescape(text.extract(att, 'href="', '"')[0]),
+                    "id"  : text.parse_int(text.extr(
+                        att, 'data-upload-id="', '"')),
+                    "name": text.unescape(text.extr(
+                        att, 'doc_preview-title">', '<')),
+                    "url" : text.unescape(text.extr(att, 'href="', '"')),
                     "type": "attachment",
                 })
 
@@ -118,8 +117,8 @@ class SubscribestarExtractor(Extractor):
             "author_id"  : text.parse_int(extr('data-user-id="', '"')),
             "author_name": text.unescape(extr('href="/', '"')),
             "author_nick": text.unescape(extr('>', '<')),
-            "date"       : self._parse_datetime(text.remove_html(extr(
-                'class="post-date">', '</'))),
+            "date"       : self._parse_datetime(extr(
+                'class="post-date">', '</').rpartition(">")[2]),
             "content"    : (extr(
                 '<div class="post-content', '<div class="post-uploads')
                 .partition(">")[2]),
@@ -152,8 +151,6 @@ class SubscribestarUserExtractor(SubscribestarExtractor):
                 "type"   : "re:image|video|attachment",
                 "url"    : str,
                 "?pinned": bool,
-                "?height": int,
-                "?width" : int,
             },
         }),
         ("https://www.subscribestar.com/subscribestar", {
@@ -162,8 +159,8 @@ class SubscribestarUserExtractor(SubscribestarExtractor):
             "range": "1",
         }),
         ("https://subscribestar.adult/kanashiipanda", {
-            "range": "21-40",
-            "count": 20,
+            "range": "1-10",
+            "count": 10,
         }),
     )
 
@@ -177,7 +174,7 @@ class SubscribestarUserExtractor(SubscribestarExtractor):
                 return
             yield from posts
 
-            url = text.extract(posts[-1], needle_next_page, '"')[0]
+            url = text.extr(posts[-1], needle_next_page, '"')
             if not url:
                 return
             page = self.request(self.root + text.unescape(url)).json()["html"]
