@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2016-2020 Mike Fährmann
+# Copyright 2016-2022 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation.
 
-"""Extract images from https://www.tumblr.com/"""
+"""Extractors for https://www.tumblr.com/"""
 
 from .common import Extractor, Message
 from .. import text, oauth, exception
@@ -14,29 +14,16 @@ from datetime import datetime, timedelta
 import re
 
 
-def _original_inline_image(url):
-    return re.sub(
-        (r"https?://(\d+\.media\.tumblr\.com(?:/[0-9a-f]+)?"
-         r"/tumblr(?:_inline)?_[^_]+)_\d+\.([0-9a-z]+)"),
-        r"https://\1_1280.\2", url
-    )
-
-
-def _original_video(url):
-    return re.sub(
-        (r"https?://((?:vt|vtt|ve)(?:\.media)?\.tumblr\.com"
-         r"/tumblr_[^_]+)_\d+\.([0-9a-z]+)"),
-        r"https://\1.\2", url
-    )
-
+BASE_PATTERN = (
+    r"(?:tumblr:(?:https?://)?([^/]+)|"
+    r"(?:https?://)?"
+    r"(?:www\.tumblr\.com/(?:blog/(?:view/)?)?([\w-]+)|"
+    r"([\w-]+\.tumblr\.com)))"
+)
 
 POST_TYPES = frozenset((
     "text", "quote", "link", "answer", "video", "audio", "photo", "chat"))
 
-BASE_PATTERN = (
-    r"(?:tumblr:(?:https?://)?([^/]+)|"
-    r"(?:https?://)?([^.]+\.tumblr\.com))")
-
 
 class TumblrExtractor(Extractor):
     """Base class for tumblr extractors"""
@@ -48,14 +35,22 @@ class TumblrExtractor(Extractor):
 
     def __init__(self, match):
         Extractor.__init__(self, match)
-        self.blog = match.group(1) or match.group(2)
-        self.api = TumblrAPI(self)
 
+        name = match.group(2)
+        if name:
+            self.blog = name + ".tumblr.com"
+        else:
+            self.blog = match.group(1) or match.group(3)
+
+        self.api = TumblrAPI(self)
         self.types = self._setup_posttypes()
         self.avatar = self.config("avatar", False)
         self.inline = self.config("inline", True)
         self.reblogs = self.config("reblogs", True)
         self.external = self.config("external", False)
+        self.original = self.config("original", True)
+        self.fallback_delay = self.config("fallback-delay", 120.0)
+        self.fallback_retries = self.config("fallback-retries", 2)
 
         if len(self.types) == 1:
             self.api.posts_type = next(iter(self.types))
@@ -70,6 +65,18 @@ class TumblrExtractor(Extractor):
     def items(self):
         blog = None
 
+        # pre-compile regular expressions
+        self._sub_video = re.compile(
+            r"https?://((?:vt|vtt|ve)(?:\.media)?\.tumblr\.com"
+            r"/tumblr_[^_]+)_\d+\.([0-9a-z]+)").sub
+        if self.inline:
+            self._sub_image = re.compile(
+                r"https?://(\d+\.media\.tumblr\.com(?:/[0-9a-f]+)?"
+                r"/tumblr(?:_inline)?_[^_]+)_\d+\.([0-9a-z]+)").sub
+            self._subn_orig_image = re.compile(r"/s\d+x\d+/").subn
+            _findall_image = re.compile('<img src="([^"]+)"').findall
+            _findall_video = re.compile('<source src="([^"]+)"').findall
+
         for post in self.posts():
             if self.date_min > post["timestamp"]:
                 return
@@ -93,8 +100,7 @@ class TumblrExtractor(Extractor):
                 del post["trail"]
             post["blog"] = blog
             post["date"] = text.parse_timestamp(post["timestamp"])
-            yield Message.Directory, post
-            post["num"] = 0
+            posts = []
 
             if "photos" in post:  # type "photo" or "link"
                 photos = post["photos"]
@@ -102,35 +108,66 @@ class TumblrExtractor(Extractor):
 
                 for photo in photos:
                     post["photo"] = photo
-                    photo.update(photo["original_size"])
+
+                    best_photo = photo["original_size"]
+                    for alt_photo in photo["alt_sizes"]:
+                        if (alt_photo["height"] > best_photo["height"] or
+                                alt_photo["width"] > best_photo["width"]):
+                            best_photo = alt_photo
+                    photo.update(best_photo)
+
+                    if self.original and "/s2048x3072/" in photo["url"] and (
+                            photo["width"] == 2048 or photo["height"] == 3072):
+                        photo["url"], fb = self._original_photo(photo["url"])
+                        if fb:
+                            post["_fallback"] = self._original_image_fallback(
+                                photo["url"], post["id"])
+
                     del photo["original_size"]
                     del photo["alt_sizes"]
-                    yield self._prepare_image(photo["url"], post)
+                    posts.append(
+                        self._prepare_image(photo["url"], post.copy()))
+                    del post["photo"]
+                    post.pop("_fallback", None)
 
             url = post.get("audio_url")  # type "audio"
             if url and url.startswith("https://a.tumblr.com/"):
-                yield self._prepare(url, post)
+                posts.append(self._prepare(url, post.copy()))
 
             url = post.get("video_url")  # type "video"
             if url:
-                yield self._prepare(_original_video(url), post)
+                posts.append(self._prepare(
+                    self._original_video(url), post.copy()))
 
             if self.inline and "reblog" in post:  # inline media
                 # only "chat" posts are missing a "reblog" key in their
                 # API response, but they can't contain images/videos anyway
                 body = post["reblog"]["comment"] + post["reblog"]["tree_html"]
-                for url in re.findall('<img src="([^"]+)"', body):
-                    url = _original_inline_image(url)
-                    yield self._prepare_image(url, post)
-                for url in re.findall('<source src="([^"]+)"', body):
-                    url = _original_video(url)
-                    yield self._prepare(url, post)
+                for url in _findall_image(body):
+                    url, fb = self._original_inline_image(url)
+                    if fb:
+                        post["_fallback"] = self._original_image_fallback(
+                            url, post["id"])
+                    posts.append(self._prepare_image(url, post.copy()))
+                    post.pop("_fallback", None)
+                for url in _findall_video(body):
+                    url = self._original_video(url)
+                    posts.append(self._prepare(url, post.copy()))
 
             if self.external:  # external links
-                post["extension"] = None
                 url = post.get("permalink_url") or post.get("url")
                 if url:
-                    yield Message.Queue, url, post
+                    post["extension"] = None
+                    posts.append((Message.Queue, url, post.copy()))
+                    del post["extension"]
+
+            post["count"] = len(posts)
+            yield Message.Directory, post
+
+            for num, (msg, url, post) in enumerate(posts, 1):
+                post["num"] = num
+                post["count"] = len(posts)
+                yield msg, url, post
 
     def posts(self):
         """Return an iterable containing all relevant posts"""
@@ -159,14 +196,21 @@ class TumblrExtractor(Extractor):
     @staticmethod
     def _prepare(url, post):
         text.nameext_from_url(url, post)
-        post["num"] += 1
         post["hash"] = post["filename"].partition("_")[2]
         return Message.Url, url, post
 
     @staticmethod
     def _prepare_image(url, post):
         text.nameext_from_url(url, post)
-        post["num"] += 1
+
+        # try ".gifv" (#3095)
+        # it's unknown whether all gifs in this case are actually webps
+        # incorrect extensions will be corrected by 'adjust-extensions'
+        if post["extension"] == "gif":
+            post["_fallback"] = (url + "v",)
+            post["_http_headers"] = {"Accept":  # copied from chrome 106
+                                     "image/avif,image/webp,image/apng,"
+                                     "image/svg+xml,image/*,*/*;q=0.8"}
 
         parts = post["filename"].split("_")
         try:
@@ -180,7 +224,7 @@ class TumblrExtractor(Extractor):
     @staticmethod
     def _prepare_avatar(url, post, blog):
         text.nameext_from_url(url, post)
-        post["num"] = 1
+        post["num"] = post["count"] = 1
         post["blog"] = blog
         post["reblogged"] = False
         post["type"] = post["id"] = post["hash"] = "avatar"
@@ -192,6 +236,37 @@ class TumblrExtractor(Extractor):
     def _skip_reblog_same_blog(self, post):
         return self.blog != post.get("reblogged_root_uuid")
 
+    def _original_photo(self, url):
+        resized = url.replace("/s2048x3072/", "/s99999x99999/", 1)
+        return self._update_image_token(resized)
+
+    def _original_inline_image(self, url):
+        if self.original:
+            resized, n = self._subn_orig_image("/s99999x99999/", url, 1)
+            if n:
+                return self._update_image_token(resized)
+        return self._sub_image(r"https://\1_1280.\2", url), False
+
+    def _original_video(self, url):
+        return self._sub_video(r"https://\1.\2", url)
+
+    def _update_image_token(self, resized):
+        headers = {"Accept": "text/html,*/*;q=0.8"}
+        try:
+            response = self.request(resized, headers=headers)
+        except Exception:
+            return resized, True
+        else:
+            updated = text.extr(response.text, '" src="', '"')
+            return updated, (resized == updated)
+
+    def _original_image_fallback(self, url, post_id):
+        for _ in range(self.fallback_retries):
+            self.sleep(self.fallback_delay, "image token")
+            yield self._update_image_token(url)[0]
+        self.log.warning("Unable to fetch higher-resolution "
+                         "version of %s (%s)", url, post_id)
+
 
 class TumblrUserExtractor(TumblrExtractor):
     """Extractor for all images from a tumblr-user"""
@@ -211,7 +286,11 @@ class TumblrUserExtractor(TumblrExtractor):
             "count": 3,
             "options": (("posts", "all"), ("external", True))
         }),
-        ("https://mikf123-hidden.tumblr.com/", {  # dashbord-only
+        ("https://mikf123-hidden.tumblr.com/", {  # dashboard-only
+            "options": (("access-token", None),),
+            "exception": exception.AuthorizationError,
+        }),
+        ("https://mikf123-hidden.tumblr.com/", {  # dashboard-only
             "count": 2,
             "keyword": {"tags": ["test", "hidden"]},
         }),
@@ -232,6 +311,9 @@ class TumblrUserExtractor(TumblrExtractor):
         ("https://demo.tumblr.com/archive"),
         ("tumblr:http://www.b-authentique.com/"),
         ("tumblr:www.b-authentique.com"),
+        ("https://www.tumblr.com/blog/view/smarties-art"),
+        ("https://www.tumblr.com/blog/smarties-art"),
+        ("https://www.tumblr.com/smarties-art"),
     )
 
     def posts(self):
@@ -241,7 +323,7 @@ class TumblrUserExtractor(TumblrExtractor):
 class TumblrPostExtractor(TumblrExtractor):
     """Extractor for images from a single post on tumblr"""
     subcategory = "post"
-    pattern = BASE_PATTERN + r"/(?:post|image)/(\d+)"
+    pattern = BASE_PATTERN + r"/(?:post/|image/)?(\d+)"
     test = (
         ("http://demo.tumblr.com/post/459265350", {
             "pattern": (r"https://\d+\.media\.tumblr\.com"
@@ -269,15 +351,32 @@ class TumblrPostExtractor(TumblrExtractor):
         ("https://mikf123.tumblr.com/post/181022380064/chat-post", {
             "count": 0,
         }),
+        ("https://kichatundk.tumblr.com/post/654953419288821760", {
+            "count": 2,  # high-quality images (#1846)
+            "content": "d6fcc7b6f750d835d55c7f31fa3b63be26c9f89b",
+        }),
+        ("https://hameru-is-cool.tumblr.com/post/639261855227002880", {
+            "count": 2,  # high-quality images (#1344)
+            "content": "6bc19a42787e46e1bba2ef4aeef5ca28fcd3cd34",
+        }),
+        ("https://mikf123.tumblr.com/image/689860196535762944", {
+            "pattern": r"^https://\d+\.media\.tumblr\.com"
+                       r"/134791621559a79793563b636b5fe2c6"
+                       r"/8f1131551cef6e74-bc/s99999x99999"
+                       r"/188cf9b8915b0d0911c6c743d152fc62e8f38491\.png$",
+        }),
         ("http://ziemniax.tumblr.com/post/109697912859/", {
             "exception": exception.NotFoundError,  # HTML response (#297)
         }),
         ("http://demo.tumblr.com/image/459265350"),
+        ("https://www.tumblr.com/blog/view/smarties-art/686047436641353728"),
+        ("https://www.tumblr.com/blog/smarties-art/686047436641353728"),
+        ("https://www.tumblr.com/smarties-art/686047436641353728"),
     )
 
     def __init__(self, match):
         TumblrExtractor.__init__(self, match)
-        self.post_id = match.group(3)
+        self.post_id = match.group(4)
         self.reblogs = True
         self.date_min = 0
 
@@ -293,14 +392,20 @@ class TumblrTagExtractor(TumblrExtractor):
     """Extractor for images from a tumblr-user by tag"""
     subcategory = "tag"
     pattern = BASE_PATTERN + r"/tagged/([^/?#]+)"
-    test = ("http://demo.tumblr.com/tagged/Times%20Square", {
-        "pattern": (r"https://\d+\.media\.tumblr\.com/tumblr_[^/_]+_1280.jpg"),
-        "count": 1,
-    })
+    test = (
+        ("http://demo.tumblr.com/tagged/Times%20Square", {
+            "pattern": r"https://\d+\.media\.tumblr\.com"
+                       r"/tumblr_[^/_]+_1280.jpg",
+            "count": 1,
+        }),
+        ("https://www.tumblr.com/blog/view/smarties-art/tagged/undertale"),
+        ("https://www.tumblr.com/blog/smarties-art/tagged/undertale"),
+        ("https://www.tumblr.com/smarties-art/tagged/undertale"),
+    )
 
     def __init__(self, match):
         TumblrExtractor.__init__(self, match)
-        self.tag = text.unquote(match.group(3).replace("-", " "))
+        self.tag = text.unquote(match.group(4).replace("-", " "))
 
     def posts(self):
         return self.api.posts(self.blog, {"tag": self.tag})
@@ -312,9 +417,14 @@ class TumblrLikesExtractor(TumblrExtractor):
     directory_fmt = ("{category}", "{blog_name}", "likes")
     archive_fmt = "f_{blog[name]}_{id}_{num}"
     pattern = BASE_PATTERN + r"/likes"
-    test = ("http://mikf123.tumblr.com/likes", {
-        "count": 1,
-    })
+    test = (
+        ("http://mikf123.tumblr.com/likes", {
+            "count": 1,
+        }),
+        ("https://www.tumblr.com/blog/view/mikf123/likes"),
+        ("https://www.tumblr.com/blog/mikf123/likes"),
+        ("https://www.tumblr.com/mikf123/likes"),
+    )
 
     def posts(self):
         return self.api.likes(self.blog)
@@ -347,11 +457,15 @@ class TumblrAPI(oauth.OAuth1API):
 
     def posts(self, blog, params):
         """Retrieve published posts"""
-        params.update({"offset": 0, "limit": 50, "reblog_info": "true"})
+        params["offset"] = self.extractor.config("offset") or 0
+        params["limit"] = 50
+        params["reblog_info"] = "true"
+
         if self.posts_type:
             params["type"] = self.posts_type
         if self.before:
             params["before"] = self.before
+
         while True:
             data = self._call(blog, "posts", params)
             self.BLOG_CACHE[blog] = data["blog"]
@@ -388,18 +502,42 @@ class TumblrAPI(oauth.OAuth1API):
             if 200 <= status < 400:
                 return data["response"]
 
+        self.log.debug(data)
         if status == 403:
             raise exception.AuthorizationError()
-        elif status == 404:
-            raise exception.NotFoundError("user or post")
-        elif status == 429:
 
+        elif status == 404:
+            try:
+                error = data["errors"][0]["detail"]
+                board = ("only viewable within the Tumblr dashboard" in error)
+            except Exception:
+                board = False
+
+            if board:
+                self.log.info("Run 'gallery-dl oauth:tumblr' "
+                              "to access dashboard-only blogs")
+                raise exception.AuthorizationError(error)
+            raise exception.NotFoundError("user or post")
+
+        elif status == 429:
             # daily rate limit
             if response.headers.get("x-ratelimit-perday-remaining") == "0":
+                self.log.info("Daily API rate limit exceeded")
                 reset = response.headers.get("x-ratelimit-perday-reset")
-                t = (datetime.now() + timedelta(seconds=float(reset))).time()
 
-                self.log.error("Daily API rate limit exceeded")
+                api_key = self.api_key or self.session.auth.consumer_key
+                if api_key == self.API_KEY:
+                    self.log.info("Register your own OAuth application and "
+                                  "use its credentials to prevent this error: "
+                                  "https://github.com/mikf/gallery-dl/blob/mas"
+                                  "ter/docs/configuration.rst#extractortumblra"
+                                  "pi-key--api-secret")
+
+                if self.extractor.config("ratelimit") == "wait":
+                    self.extractor.wait(seconds=reset)
+                    return self._call(blog, endpoint, params)
+
+                t = (datetime.now() + timedelta(seconds=float(reset))).time()
                 raise exception.StopExtraction(
                     "Aborting - Rate limit will reset at %s",
                     "{:02}:{:02}:{:02}".format(t.hour, t.minute, t.second))
