@@ -41,6 +41,10 @@ class TwitterExtractor(Extractor):
         self.cards = self.config("cards", False)
         self.cards_blacklist = self.config("cards-blacklist")
         self.syndication = self.config("syndication")
+
+        if not self.config("transform", True):
+            self._transform_user = util.identity
+            self._transform_tweet = util.identity
         self._user = self._user_obj = None
         self._user_cache = {}
         self._init_sizes()
@@ -212,7 +216,7 @@ class TwitterExtractor(Extractor):
                             files.append(value)
                             return
         elif name == "unified_card":
-            data = json.loads(bvals["unified_card"]["string_value"])
+            data = util.json_loads(bvals["unified_card"]["string_value"])
             self._extract_media(tweet, data["media_entities"].values(), files)
             return
 
@@ -1436,6 +1440,8 @@ class TwitterAPI():
 
                 if "retweeted_status_result" in legacy:
                     retweet = legacy["retweeted_status_result"]["result"]
+                    if "tweet" in retweet:
+                        retweet = retweet["tweet"]
                     if original_retweets:
                         try:
                             retweet["legacy"]["retweeted_status_id_str"] = \
