@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2021-2022 Mike Fährmann
+# Copyright 2021-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -79,7 +79,8 @@ class VkExtractor(Extractor):
 
             if len(payload) < 4:
                 self.log.debug(payload)
-                raise exception.AuthorizationError(payload[0])
+                raise exception.AuthorizationError(
+                    text.unescape(payload[0]) if payload[0] else None)
 
             total = payload[1]
             photos = payload[3]
@@ -103,13 +104,13 @@ class VkPhotosExtractor(VkExtractor):
     subcategory = "photos"
     pattern = (BASE_PATTERN + r"/(?:"
                r"(?:albums|photos|id)(-?\d+)"
-               r"|(?!album-?\d+_)([^/?#]+))")
+               r"|(?!(?:album|tag)-?\d+_?)([^/?#]+))")
     test = (
         ("https://vk.com/id398982326", {
             "pattern": r"https://sun\d+-\d+\.userapi\.com/s/v1/if1"
                        r"/[\w-]+\.jpg\?size=\d+x\d+&quality=96&type=album",
             "count": ">= 35",
-            "keywords": {
+            "keyword": {
                 "id": r"re:\d+",
                 "user": {
                     "id": "398982326",
@@ -121,12 +122,11 @@ class VkPhotosExtractor(VkExtractor):
         }),
         ("https://vk.com/cosplayinrussia", {
             "range": "15-25",
-            "keywords": {
+            "keyword": {
                 "id": r"re:\d+",
                 "user": {
                     "id"  : "-165740836",
-                    "info": "Предложка открыта, кидайте ваши косплейчики. При "
-                            "правильном оформлении они будут опубликованы",
+                    "info": str,
                     "name": "cosplayinrussia",
                     "nick": "Косплей | Cosplay 18+",
                 },
@@ -182,9 +182,6 @@ class VkAlbumExtractor(VkExtractor):
     directory_fmt = ("{category}", "{user[id]}", "{album[id]}")
     pattern = BASE_PATTERN + r"/album(-?\d+)_(\d+)$"
     test = (
-        ("https://vk.com/album232175027_00", {
-            "count": 8,
-        }),
         ("https://vk.com/album-165740836_281339889", {
             "count": 12,
         }),
@@ -192,6 +189,9 @@ class VkAlbumExtractor(VkExtractor):
         ("https://vk.com/album-53775183_00", {
             "exception": exception.AuthorizationError,
         }),
+        ("https://vk.com/album232175027_00", {
+            "exception": exception.AuthorizationError,
+        }),
     )
 
     def __init__(self, match):
@@ -207,3 +207,25 @@ class VkAlbumExtractor(VkExtractor):
             "user": {"id": self.user_id},
             "album": {"id": self.album_id},
         }
+
+
+class VkTaggedExtractor(VkExtractor):
+    """Extractor for a vk tagged photos"""
+    subcategory = "tagged"
+    directory_fmt = ("{category}", "{user[id]}", "tags")
+    pattern = BASE_PATTERN + r"/tag(-?\d+)$"
+    test = (
+        ("https://vk.com/tag304303884", {
+            "count": 44,
+        }),
+    )
+
+    def __init__(self, match):
+        VkExtractor.__init__(self, match)
+        self.user_id = match.group(1)
+
+    def photos(self):
+        return self._pagination("tag{}".format(self.user_id))
+
+    def metadata(self):
+        return {"user": {"id": self.user_id}}
