@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2015-2021 Mike Fährmann
+# Copyright 2015-2023 Mike Fährmann
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -10,6 +10,7 @@ import os
 import sys
 import shutil
 import logging
+import functools
 import unicodedata
 from . import config, util, formatter
 
@@ -23,7 +24,7 @@ LOG_LEVEL = logging.INFO
 
 
 class Logger(logging.Logger):
-    """Custom logger that includes extra info in log records"""
+    """Custom Logger that includes extra info in log records"""
 
     def makeRecord(self, name, level, fn, lno, msg, args, exc_info,
                    func=None, extra=None, sinfo=None,
@@ -38,9 +39,9 @@ class LoggerAdapter():
     """Trimmed-down version of logging.LoggingAdapter"""
     __slots__ = ("logger", "extra")
 
-    def __init__(self, logger, extra):
+    def __init__(self, logger, job):
         self.logger = logger
-        self.extra = extra
+        self.extra = job._logger_extra
 
     def debug(self, msg, *args, **kwargs):
         if self.logger.isEnabledFor(logging.DEBUG):
@@ -63,6 +64,38 @@ class LoggerAdapter():
             self.logger._log(logging.ERROR, msg, args, **kwargs)
 
 
+class LoggerAdapterActions():
+
+    def __init__(self, logger, job):
+        self.logger = logger
+        self.extra = job._logger_extra
+        self.actions = job._logger_actions
+
+        self.debug = functools.partial(self.log, logging.DEBUG)
+        self.info = functools.partial(self.log, logging.INFO)
+        self.warning = functools.partial(self.log, logging.WARNING)
+        self.error = functools.partial(self.log, logging.ERROR)
+
+    def log(self, level, msg, *args, **kwargs):
+        if args:
+            msg = msg % args
+
+        actions = self.actions[level]
+        if actions:
+            args = self.extra.copy()
+            args["level"] = level
+
+            for cond, action in actions:
+                if cond(msg):
+                    action(args)
+
+            level = args["level"]
+
+        if self.logger.isEnabledFor(level):
+            kwargs["extra"] = self.extra
+            self.logger._log(level, msg, (), **kwargs)
+
+
 class PathfmtProxy():
     __slots__ = ("job",)
 
@@ -73,6 +106,12 @@ class PathfmtProxy():
         pathfmt = object.__getattribute__(self, "job").pathfmt
         return pathfmt.__dict__.get(name) if pathfmt else None
 
+    def __str__(self):
+        pathfmt = object.__getattribute__(self, "job").pathfmt
+        if pathfmt:
+            return pathfmt.path or pathfmt.directory
+        return ""
+
 
 class KwdictProxy():
     __slots__ = ("job",)
@@ -205,16 +244,56 @@ def setup_logging_handler(key, fmt=LOG_FORMAT, lvl=LOG_LEVEL):
 # --------------------------------------------------------------------
 # Utility functions
 
-def replace_std_streams(errors="replace"):
-    """Replace standard streams and set their error handlers to 'errors'"""
-    for name in ("stdout", "stdin", "stderr"):
-        stream = getattr(sys, name)
-        if stream:
+def stdout_write_flush(s):
+    sys.stdout.write(s)
+    sys.stdout.flush()
+
+
+def stderr_write_flush(s):
+    sys.stderr.write(s)
+    sys.stderr.flush()
+
+
+if sys.stdout.line_buffering:
+    def stdout_write(s):
+        sys.stdout.write(s)
+else:
+    stdout_write = stdout_write_flush
+
+
+if sys.stderr.line_buffering:
+    def stderr_write(s):
+        sys.stderr.write(s)
+else:
+    stderr_write = stderr_write_flush
+
+
+def configure_standard_streams():
+    for name in ("stdout", "stderr", "stdin"):
+        options = config.get(("output",), name)
+        if not options:
+            continue
+
+        stream = getattr(sys, name, None)
+        if not stream:
+            continue
+
+        if isinstance(options, str):
+            options = {"encoding": options, "errors": "replace"}
+        elif not options.get("errors"):
+            options["errors"] = "replace"
+
+        try:
+            stream.reconfigure(**options)
+        except AttributeError:
+            # no 'reconfigure' support
+            oget = options.get
             setattr(sys, name, stream.__class__(
                 stream.buffer,
-                errors=errors,
-                newline=stream.newlines,
-                line_buffering=stream.line_buffering,
+                encoding=oget("encoding", stream.encoding),
+                errors=oget("errors", "replace"),
+                newline=oget("newline", stream.newlines),
+                line_buffering=oget("line_buffering", stream.line_buffering),
             ))
 
 
@@ -222,25 +301,25 @@ def replace_std_streams(errors="replace"):
 # Downloader output
 
 def select():
-    """Automatically select a suitable output class"""
-    pdict = {
-        "default": PipeOutput,
-        "pipe": PipeOutput,
-        "term": TerminalOutput,
-        "terminal": TerminalOutput,
-        "color": ColorOutput,
-        "null": NullOutput,
-    }
-    omode = config.get(("output",), "mode", "auto").lower()
-    if omode in pdict:
-        output = pdict[omode]()
-    elif omode == "auto":
+    """Select a suitable output class"""
+    mode = config.get(("output",), "mode")
+
+    if mode is None or mode == "auto":
         if hasattr(sys.stdout, "isatty") and sys.stdout.isatty():
             output = ColorOutput() if ANSI else TerminalOutput()
         else:
             output = PipeOutput()
+    elif isinstance(mode, dict):
+        output = CustomOutput(mode)
     else:
-        raise Exception("invalid output mode: " + omode)
+        output = {
+            "default" : PipeOutput,
+            "pipe"    : PipeOutput,
+            "term"    : TerminalOutput,
+            "terminal": TerminalOutput,
+            "color"   : ColorOutput,
+            "null"    : NullOutput,
+        }[mode.lower()]()
 
     if not config.get(("output",), "skip", True):
         output.skip = util.identity
@@ -255,7 +334,7 @@ class NullOutput():
     def skip(self, path):
         """Print a message indicating that a download has been skipped"""
 
-    def success(self, path, tries):
+    def success(self, path):
         """Print a message indicating the completion of a download"""
 
     def progress(self, bytes_total, bytes_downloaded, bytes_per_second):
@@ -265,17 +344,13 @@ class NullOutput():
 class PipeOutput(NullOutput):
 
     def skip(self, path):
-        stdout = sys.stdout
-        stdout.write(CHAR_SKIP + path + "\n")
-        stdout.flush()
+        stdout_write(CHAR_SKIP + path + "\n")
 
-    def success(self, path, tries):
-        stdout = sys.stdout
-        stdout.write(path + "\n")
-        stdout.flush()
+    def success(self, path):
+        stdout_write(path + "\n")
 
 
-class TerminalOutput(NullOutput):
+class TerminalOutput():
 
     def __init__(self):
         shorten = config.get(("output",), "shorten", True)
@@ -288,38 +363,105 @@ class TerminalOutput(NullOutput):
             self.shorten = util.identity
 
     def start(self, path):
-        stdout = sys.stdout
-        stdout.write(self.shorten("  " + path))
-        stdout.flush()
+        stdout_write_flush(self.shorten("  " + path))
 
     def skip(self, path):
-        sys.stdout.write(self.shorten(CHAR_SKIP + path) + "\n")
+        stdout_write(self.shorten(CHAR_SKIP + path) + "\n")
 
-    def success(self, path, tries):
-        sys.stdout.write("\r" + self.shorten(CHAR_SUCCESS + path) + "\n")
+    def success(self, path):
+        stdout_write("\r" + self.shorten(CHAR_SUCCESS + path) + "\n")
 
     def progress(self, bytes_total, bytes_downloaded, bytes_per_second):
         bdl = util.format_value(bytes_downloaded)
         bps = util.format_value(bytes_per_second)
         if bytes_total is None:
-            sys.stderr.write("\r{:>7}B {:>7}B/s ".format(bdl, bps))
+            stderr_write("\r{:>7}B {:>7}B/s ".format(bdl, bps))
         else:
-            sys.stderr.write("\r{:>3}% {:>7}B {:>7}B/s ".format(
+            stderr_write("\r{:>3}% {:>7}B {:>7}B/s ".format(
                 bytes_downloaded * 100 // bytes_total, bdl, bps))
 
 
 class ColorOutput(TerminalOutput):
 
+    def __init__(self):
+        TerminalOutput.__init__(self)
+
+        colors = config.get(("output",), "colors") or {}
+        self.color_skip = "\033[{}m".format(
+            colors.get("skip", "2"))
+        self.color_success = "\r\033[{}m".format(
+            colors.get("success", "1;32"))
+
     def start(self, path):
-        stdout = sys.stdout
-        stdout.write(self.shorten(path))
-        stdout.flush()
+        stdout_write_flush(self.shorten(path))
 
     def skip(self, path):
-        sys.stdout.write("\033[2m" + self.shorten(path) + "\033[0m\n")
+        stdout_write(self.color_skip + self.shorten(path) + "\033[0m\n")
 
-    def success(self, path, tries):
-        sys.stdout.write("\r\033[1;32m" + self.shorten(path) + "\033[0m\n")
+    def success(self, path):
+        stdout_write(self.color_success + self.shorten(path) + "\033[0m\n")
+
+
+class CustomOutput():
+
+    def __init__(self, options):
+
+        fmt_skip = options.get("skip")
+        fmt_start = options.get("start")
+        fmt_success = options.get("success")
+        off_skip = off_start = off_success = 0
+
+        if isinstance(fmt_skip, list):
+            off_skip, fmt_skip = fmt_skip
+        if isinstance(fmt_start, list):
+            off_start, fmt_start = fmt_start
+        if isinstance(fmt_success, list):
+            off_success, fmt_success = fmt_success
+
+        shorten = config.get(("output",), "shorten", True)
+        if shorten:
+            func = shorten_string_eaw if shorten == "eaw" else shorten_string
+            width = shutil.get_terminal_size().columns
+
+            self._fmt_skip = self._make_func(
+                func, fmt_skip, width - off_skip)
+            self._fmt_start = self._make_func(
+                func, fmt_start, width - off_start)
+            self._fmt_success = self._make_func(
+                func, fmt_success, width - off_success)
+        else:
+            self._fmt_skip = fmt_skip.format
+            self._fmt_start = fmt_start.format
+            self._fmt_success = fmt_success.format
+
+        self._fmt_progress = (options.get("progress") or
+                              "\r{0:>7}B {1:>7}B/s ").format
+        self._fmt_progress_total = (options.get("progress-total") or
+                                    "\r{3:>3}% {0:>7}B {1:>7}B/s ").format
+
+    @staticmethod
+    def _make_func(shorten, format_string, limit):
+        fmt = format_string.format
+        return lambda txt: fmt(shorten(txt, limit, CHAR_ELLIPSIES))
+
+    def start(self, path):
+        stdout_write_flush(self._fmt_start(path))
+
+    def skip(self, path):
+        stdout_write(self._fmt_skip(path))
+
+    def success(self, path):
+        stdout_write(self._fmt_success(path))
+
+    def progress(self, bytes_total, bytes_downloaded, bytes_per_second):
+        bdl = util.format_value(bytes_downloaded)
+        bps = util.format_value(bytes_per_second)
+        if bytes_total is None:
+            stderr_write(self._fmt_progress(bdl, bps))
+        else:
+            stderr_write(self._fmt_progress_total(
+                bdl, bps, util.format_value(bytes_total),
+                bytes_downloaded * 100 // bytes_total))
 
 
 class EAWCache(dict):
