@@ -39,6 +39,16 @@ def split_html(txt):
         return []
 
 
+def slugify(value):
+    """Convert a string to a URL slug
+
+    Adapted from:
+    https://github.com/django/django/blob/master/django/utils/text.py
+    """
+    value = re.sub(r"[^\w\s-]", "", str(value).lower())
+    return re.sub(r"[-\s]+", "-", value).strip("-_")
+
+
 def ensure_http_scheme(url, scheme="https://"):
     """Prepend 'scheme' to 'url' if it doesn't have one"""
     if url and not url.startswith(("https://", "http://")):
@@ -110,6 +120,15 @@ def extract(txt, begin, end, pos=0):
         return None, pos
 
 
+def extr(txt, begin, end, default=""):
+    """Stripped-down version of 'extract()'"""
+    try:
+        first = txt.index(begin) + len(begin)
+        return txt[first:txt.index(end, first)]
+    except (ValueError, TypeError, AttributeError):
+        return default
+
+
 def rextract(txt, begin, end, pos=-1):
     try:
         lbeg = len(begin)
