@@ -312,6 +312,7 @@ def setup_test_config():
 
     config.set(("extractor", "nijie")     , "username", email)
     config.set(("extractor", "seiga")     , "username", email)
+    config.set(("extractor", "horne")     , "username", email2)
     config.set(("extractor", "pinterest") , "username", email2)
     config.set(("extractor", "pinterest") , "username", None)  # login broken
 
@@ -321,14 +322,18 @@ def setup_test_config():
     config.set(("extractor", "mangoxo")   , "username", "LiQiang3")
     config.set(("extractor", "mangoxo")   , "password", "5zbQF10_5u25259Ma")
 
-    for category in ("danbooru", "instagram", "twitter", "subscribestar",
-                     "e621", "atfbooru", "inkbunny", "tapas", "pillowfort",
-                     "mangadex"):
+    for category in ("danbooru", "atfbooru", "aibooru", "e621", "e926",
+                     "instagram", "twitter", "subscribestar", "deviantart",
+                     "inkbunny", "tapas", "pillowfort", "mangadex"):
         config.set(("extractor", category), "username", None)
 
     config.set(("extractor", "mastodon.social"), "access-token",
                "Blf9gVqG7GytDTfVMiyYQjwVMQaNACgf3Ds3IxxVDUQ")
 
+    config.set(("extractor", "nana"), "favkey",
+               "9237ddb82019558ea7d179e805100805"
+               "ea6aa1c53ca6885cd4c179f9fb22ead2")
+
     config.set(("extractor", "deviantart"), "client-id", "7777")
     config.set(("extractor", "deviantart"), "client-secret",
                "ff14994c744d9208e5caeec7aab4a026")
